(*----------------------------------------------------------------------------*
 *  Direct3D VCLfullscreenD3D9                                                *
 *  shows how to use Delphi VCL dialogs in Direct3D 9.0 fullscreen mode       *
 *                                                                            *
 *  Note: DirectX 9.0 Summer 2003 SDK compatible frawework required           *
 *                                                                            *
 *  Copyright (c) 2004 by Alexey Barkovoy                                     *
 *  E-Mail: clootie@ixbt.com                                                  *
 *                                                                            *
 *  Modified: 26-Jan-2004                                                     *
 *                                                                            *
 *  Latest version can be downloaded from:                                    *
 *     http://clootie.narod.ru/delphi/download_custom.html                    *
 *----------------------------------------------------------------------------*)
// Revision 1.1 -- modified for Summer2003 SDK

unit MainUnit;

interface

uses
  Windows, Messages, Classes, SysUtils,
  Forms, Dialogs, Menus, Controls, StdCtrls, ComCtrls, ExtCtrls, AppEvnts,
  D3DX9, DXFile, D3Dpart;

type
  CMyD3DApplication2 = class(CMyD3DApplication) end;

  TMainForm = class(TForm)
    DisplayPanel: TPanel;
    Splitter1: TSplitter;
    MainMenu1: TMainMenu;
    StatusBar: TStatusBar;
    File1: TMenuItem;
    Exit1: TMenuItem;
    N1: TMenuItem;
    About1: TMenuItem;
    OpenDialog: TOpenDialog;
    ApplicationEvents1: TApplicationEvents;
    SaveDialog: TSaveDialog;
    ButtonPanel: TPanel;
    Splitter2: TSplitter;
    ShowDlgButton: TButton;
    FullScreenButton: TButton;
    FullScreen: TMenuItem;
    ShowDlg1: TMenuItem;
    Label1: TLabel;
    procedure FormShow(Sender: TObject);
    procedure OnToggleFullscreen(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure ApplicationEvents1Idle(Sender: TObject; var Done: Boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure About1Click(Sender: TObject);
    procedure DisplayPanelResize(Sender: TObject);
    procedure ShowDlgButtonClick(Sender: TObject);
    procedure FullScreenButtonClick(Sender: TObject);
  private
    { Private declarations }
    fMyApp: CMyD3DApplication2;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

///////////////////////////////////////////////////////////////////////////

const
  sAppName = 'Clootie - Test FullScreen Dialogs';

implementation

uses DemoDlgUnit, D3DApp;

{$R *.dfm}


//-----------------------------------------------------------------------------
// Name: FullScreenWndProc()
// Desc: The WndProc funtion used when the app is in fullscreen mode. This is
//       needed simply to trap the ESC key.
//-----------------------------------------------------------------------------
function FullScreenWndProc(hWnd: HWND; msg: UINT; wParam: WPARAM;
  lParam: LPARAM): LRESULT; stdcall;
begin
  if (msg = WM_CLOSE) then
  begin
    // User wants to exit, so go back to windowed mode and exit for real
    MainForm.OnToggleFullScreen(nil);
    PostMessage(Application.Handle, WM_CLOSE, 0, 0);
  end;

  if (msg = WM_SETCURSOR) then SetCursor(0);

  // User wants to leave fullscreen mode
  if (msg = WM_KEYUP) and (wParam = VK_ESCAPE) then
    MainForm.OnToggleFullScreen(nil);

  if (msg = WM_KEYUP) and (wParam = VK_SPACE) then
    MainForm.ShowDlgButtonClick(nil);

  if Assigned(MainForm) and Assigned(MainForm.fMyApp) then
    MainForm.fMyApp.m_ArcBall.HandleMessages(hWnd, Msg, wParam, lParam);

  Result:= DefWindowProc(hWnd, msg, wParam, lParam);
end;

procedure TMainForm.FormShow(Sender: TObject);
{$WRITEABLECONST ON}
const
  wndClass: TWndClass = (
    style: CS_HREDRAW or CS_VREDRAW;
    lpfnWndProc: @FullScreenWndProc; cbClsExtra: 0; cbWndExtra: 0; hInstance: 0;
    hIcon: 0; hCursor: 0; hbrBackground: 0; lpszMenuName: nil;
    lpszClassName: 'Fullscreen Window'
  );
{$WRITEABLECONST OFF}
begin
  OpenDialog.InitialDir:= ExtractFileDir(ParamStr(0));
  SaveDialog.InitialDir:= ExtractFileDir(ParamStr(0));

  fMyApp:= CMyD3DApplication2.Create;
  fMyApp.fAppForm:= Self;
  fMyApp.m_hwndRenderWindow:= DisplayPanel.Handle; // Save static reference to the render window

  // Register a class for a fullscreen window
  wndClass.hbrBackground:= GetStockObject(WHITE_BRUSH);
  Windows.RegisterClass(wndClass);

  // We create the fullscreen window (not visible) at startup, so it can
  // be the focus window.  The focus window can only be set at CreateDevice
  // time, not in a Reset, so ToggleFullscreen wouldn't work unless we have
  // already set up the fullscreen focus window.
  fMyApp.m_hwndRenderFullScreen:= CreateWindow('Fullscreen Window', nil,
                                      WS_POPUP, Integer(CW_USEDEFAULT),
                                      Integer(CW_USEDEFAULT), 100, 100,
                                      Application.Handle, 0, 0, nil);

  fMyApp.AssignWindowsHandles;
  fMyApp.AppCreate(HInstance);
end;

//-----------------------------------------------------------------------------
// Name: OnToggleFullScreen()
// Desc: Called when user toggles the fullscreen mode
//-----------------------------------------------------------------------------
procedure TMainForm.OnToggleFullscreen(Sender: TObject);
begin
  fMyApp.ToggleFullscreen;
end;

procedure TMainForm.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.ApplicationEvents1Idle(Sender: TObject;
  var Done: Boolean);
var
  strStatsPrev: String;
begin
  Done:= True;

  // Do not render if the app is minimized
  if IsIconic(Application.Handle) then Exit;

  // Update and render a frame
  if fMyApp.m_bActive then
  begin
    strStatsPrev:= fMyApp.m_strFrameStats;
    if not fMyApp.m_bWindowed and FAILED(fMyApp.m_pd3dDevice.TestCooperativeLevel) then
    begin
      if Assigned(DemoDlg) then
      begin // Special mode then Dialog is activated
        DemoDlg.Close;
        Exit;
      end else
      begin
        fMyApp.ForceWindowed;
        SetFocus;
        Application.BringToFront;
      end;
    end;
    fMyApp.Render3DEnvironment;
    if (strStatsPrev <> fMyApp.m_strFrameStats) then
      StatusBar.Panels[0].Text:= fMyApp.m_strFrameStats;
  end;

  // Keep requesting more idle time
  Done:= False;
end;

procedure TMainForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  DestroyWindow(fMyApp.m_hwndRenderFullScreen);
  fMyApp.m_hwndRenderFullScreen:= 0;

  FreeAndNil(fMyApp);
end;

procedure TMainForm.About1Click(Sender: TObject);
begin
  Application.MessageBox('This demo shows how to use VCL forms in fullscreen DirectX mode.'#10+
    'This and other demos can be found at http://clootie.narod.ru',
    'Clootie - Test FullScreen Dialogs', MB_ICONINFORMATION or MB_OK);
end;

procedure TMainForm.DisplayPanelResize(Sender: TObject);
begin
  if Assigned(fMyApp) and fMyApp.m_bActive then
  begin
    fMyApp.Pause(True);
    fMyApp.MsgProc(0, WM_EXITSIZEMOVE, 0, 0);
  end;
end;

procedure TMainForm.ShowDlgButtonClick(Sender: TObject);
begin
  // This is recommended by DirectX documentation (more correct handling of loosing FullScreen mode)
  if fMyApp.m_bWindowed
    then DemoDlgParentWndHandle:= 0
    else DemoDlgParentWndHandle:= fMyApp.m_hwndRenderFullScreen;

  Application.CreateForm(TDemoDlg, DemoDlg);
  try
    DemoDlg.ShowModal;
  finally
    DemoDlg.Free;
    DemoDlg:= nil;
    if not fMyApp.m_bWindowed and FAILED(fMyApp.m_pd3dDevice.TestCooperativeLevel) then
    begin
      fMyApp.ForceWindowed;
      SetFocus;
      Application.BringToFront;
    end;
  end;
end;

procedure TMainForm.FullScreenButtonClick(Sender: TObject);
begin
  OnToggleFullscreen(nil);
end;

end.

